/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include "Random.h"
#include "AllelePDF.h"


using namespace std;


AllelePDF::AllelePDF(size_t numAlleles)
: pdf(numAlleles) {
	for (size_t i = 0; i < pdf.size(); i++)
		pdf[i] = 0;
}

// Only valid after prepareForSampling has been called
size_t AllelePDF::frequency(Allele a) const throw(out_of_range) {
	if (a == 0)
		return pdf.at(a);
	else
		return pdf.at(a) - pdf.at(a-1);
}

// Only valid after prepareForSampling has been called
#ifndef THREADED
Allele AllelePDF::sample() const throw(domain_error) {
#else
Allele AllelePDF::sample(Random* random) const throw(domain_error) {
#endif
	if (pdf.empty())
		throw domain_error("AllelePDF::sample() domain_error: no samples");
#ifndef THREADED
	size_t rand = g_Random.next32(1, pdf[pdf.size()-1]);
#else
	size_t rand = random->next32(1, pdf[pdf.size()-1]);
#endif
	Allele low = 0, high = pdf.size()-1, mid;
	while (low < high) {
		mid = low + (high - low) / 2;
		if (pdf[mid] < rand)
			low = mid + 1;
		else
			high = mid;
	}
	assert(pdf[low] >= rand && (low == 0 || pdf[low-1] < rand));
	return low;
}

void AllelePDF::prepareForSampling() {
	for (Allele i = 1; i < pdf.size(); i++)
		pdf[i] += pdf[i-1];
}

void AllelePDF::clear() {
	for (Allele i = 0; i < pdf.size(); i++)
		pdf[i] = 0;
}
