/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _ALLELEPDF_H_
#define _ALLELEPDF_H_


#include "Chromosome.h"
#ifdef THREADED
#include "Random.h"
#endif


//
// Stores a probability distribution function of alleles.
//

class AllelePDF {
public:
	AllelePDF(size_t numAlleles);

	size_t frequency(Allele a) const throw(std::out_of_range);
	size_t totalFrequency() const throw(std::domain_error);
#ifndef THREADED
	Allele sample() const throw(std::domain_error);
#else
	Allele sample(Random* random) const throw(std::domain_error);
#endif

	void add(Allele a) throw(std::out_of_range);
	void prepareForSampling();
	void clear();

private:
	std::vector<size_t> pdf;
};


// Only valid after prepareForSampling has been called
inline size_t AllelePDF::totalFrequency() const throw(std::domain_error) {
	if (pdf.empty())
		throw std::domain_error("AllelePDF::totalFrequency() domain_error: no samples");
	return pdf.back();
}

inline void AllelePDF::add(Allele a) throw(std::out_of_range) {
	pdf.at(a)++;
}


#endif
