/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <algorithm>
#include <fstream>
#include <sstream>
#include "AllelePool.h"


using namespace std;


Allele AllelePool::get(const string& value) const throw(invalid_argument) {
	map<string,Allele>::const_iterator it = stringToAllele.find(value);
	if (it == stringToAllele.end()) {
		stringstream ss;
		ss << "AllelePool::get() invalid_argument: " << value;
		throw invalid_argument(ss.str());
	}
	return it->second;
}

const string& AllelePool::get(Allele a) const throw(out_of_range) {
	map<Allele,string>::const_iterator it = alleleToString.find(a);
	if (it == alleleToString.end()) {
		stringstream ss;
		ss << "AllelePool::get() out_of_range: " << a;
		throw out_of_range(ss.str());
	}
	return it->second;
}

Allele AllelePool::highestAllele() const throw(domain_error) {
	if (highestFreeAllele == 0)
		throw domain_error("AllelePool::highestAllele() domain_error");
	return highestFreeAllele - 1;
}

Allele AllelePool::newAllele(const string& value) throw(invalid_argument) {
	set(highestFreeAllele, value);
	return highestFreeAllele - 1;
}

istream& operator>>(istream& in, AllelePool& ap) {
	string name;
	stringstream ss;

	getline(in, ss);
	ss >> ws;
	while (ss.good()) {
		ss >> name;
		ap.newAllele(name);
		ss >> ws;
	}
	return in;
}

ostream& operator<<(ostream& out, const AllelePool& ap) {
	map<string,Allele>::const_iterator it = ap.stringToAllele.begin();
	if (it != ap.stringToAllele.end()) {
		out << it->first;
		it++;
	}
	while (it != ap.stringToAllele.end()) {
		out << " " << it->first;
		it++;
	}
	return out;
}

void AllelePool::set(Allele a, const string& value) throw(invalid_argument) {
	if (contains(a) || contains(value)) {
		stringstream ss;
		ss << "AllelePool::set() invalid_argument: " << a << ", " << value;
		throw invalid_argument(ss.str());
	}
	alleleToString.insert(make_pair(a, value));
	stringToAllele.insert(make_pair(value, a));
	if (a >= highestFreeAllele)
		highestFreeAllele = a + 1;
}
