/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _CHI2APPROX_H_
#define _CHI2APPROX_H_


#include <vector>
#include "util.h"


class Chi2Approx {
public:
	Chi2Approx();
	Chi2Approx(const char* filename) throw(std::runtime_error);

	double cdf(unsigned int DoF, double chi2) const throw(std::out_of_range);

	bool load(const char* filename);
	friend std::istream& operator>>(std::istream& in, Chi2Approx& ca);

#ifdef SELFTEST
	static void selfTest();
#endif

private:
	struct entry {
		entry();
		entry(double chi2, double cdf);

		double chi2, cdf;
	};

	std::vector<std::vector<entry> > levels;
};


inline Chi2Approx::Chi2Approx() {
}

inline bool Chi2Approx::load(const char* filename) {
	return ::load(filename, this);
}

inline Chi2Approx::entry::entry() {
}

inline Chi2Approx::entry::entry(double chi2, double cdf)
: chi2(chi2), cdf(cdf) {
}


extern Chi2Approx g_Chi2Approx;


#endif
