/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <fstream>
#include <new>
#include <sstream>
#include "Chromosome.h"


using namespace std;


bool Chromosome::write(ostream& out, const AllelePool& ap) const {
	out << alleles.size();
	for (Gene i = 0; i < alleles.size(); i++) {
		assert(ap.contains(alleles[i]));
		out << ' ' << ap.get(alleles[i]);
	}
	out << endl;
	return !out.bad() && !out.fail();
}

Chromosome* Chromosome::read(istream& in, const AllelePool& ap) {
	stringstream ss;
	string line;
	size_t size;

	getline(in, ss);
	ss >> size;
	Chromosome *c = new Chromosome(size);
	
	for (Gene i = 0; i < size; i++) {
		ss >> line;
		if (ss.fail() || ss.bad())
			return false;
		if (!ap.contains(line)) {
			cerr << "Error: Unknown allele symbol: " << line << endl;;
			return false;
		}
		(*c)[i] = ap.get(line);
	}
	return c;
}
