/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _CHROMOSOME_H_
#define _CHROMOSOME_H_


#include <cassert>
#include <vector>
#include "AllelePool.h"


typedef size_t Gene;


//
// A list of alleles.
//

class Chromosome {
public:
	Chromosome(size_t size);

	size_t size() const;

#ifndef NDEBUG
	Allele& operator[](Gene gene) throw(std::out_of_range);
	Allele operator[](Gene gene) const throw(std::out_of_range);
#else
	Allele& operator[](Gene gene);
	Allele operator[](Gene gene) const;
#endif
	bool write(std::ostream& out, const AllelePool& ap) const;
	static Chromosome* read(std::istream& in, const AllelePool& ap);

private:
	std::vector<Allele> alleles;
};


inline Chromosome::Chromosome(size_t size)
: alleles(size) {
	assert(size > 0);
}

inline size_t Chromosome::size() const {
	return alleles.size();
}

#ifndef NDEBUG
inline Allele& Chromosome::operator[](Gene gene) throw(std::out_of_range) {
	return alleles.at(gene);
}

inline Allele Chromosome::operator[](Gene gene) const throw(std::out_of_range) {
	return alleles.at(gene);
}
#else
inline Allele& Chromosome::operator[](Gene gene) {
	return alleles[gene];
}

inline Allele Chromosome::operator[](Gene gene) const {
	return alleles[gene];
}
#endif


#endif
