/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <cassert>
#include "FISWrapper.h"


using namespace std;


#ifdef SELFTEST
void FISWrapper::selfTest() {
	FISWrapper set(10, 5);

	assert(set.isValid());
	assert(set.numberOfMembers() == 0);
	assert(set.numberOfNonMembers() == 9);

	bool exceptionThrown = false;
	try {
		set.addMember(5);
	} catch (out_of_range& e) {
		exceptionThrown = true;
	}
	assert(exceptionThrown);
	exceptionThrown = false;
	try {
		set.isMember(11);
	} catch (out_of_range& e) {
		exceptionThrown = true;
	}
	assert(exceptionThrown);
	exceptionThrown = false;
	try {
		set.randomMember();
	} catch (domain_error& e) {
		exceptionThrown = true;
	}
	assert(exceptionThrown);
	
	for (int i = 0; i < 1000; i++)
		assert(set.randomNonMember() != 5);

	for (int i = 0; i <= 4; i++)
		set.addMember(i);
	assert(set.isValid());
	assert(set.numberOfMembers() == 5);
	assert(set.numberOfNonMembers() == 4);

	for (int i = 0; i < 1000; i++) {
		assert(set.isMember(set.randomMember()));
		assert(!set.isMember(set.randomNonMember()));
	}

	while (set.numberOfNonMembers() > 0)
		set.addMember(set.randomNonMember());
	assert(set.isValid());
	assert(set.numberOfMembers() == 9);
	assert(set.numberOfNonMembers() == 0);

	while (set.numberOfMembers() > 0)
		set.removeMember(set.randomMember());
	assert(set.isValid());
	assert(set.numberOfMembers() == 0);
	assert(set.numberOfNonMembers() == 9);
}

#endif
