/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <algorithm>
#include <fstream>
#include "Population.h"


using namespace std;


Population::~Population() {
	for (size_t i = 0; i < entries.size(); i++)
		delete entries[i].chromosome;
}

void Population::sort() {
	std::sort(entries.begin(), entries.end(), sortFunc);
}

bool Population::load(const char* filename, const AllelePool& ap) {
	ifstream in(filename);
	if (!in || in.is_open() == 0)
		return false;

	size_t size;
	in >> size;
	entries.resize(size);
	for (size_t i = 0; i < size; i++) {
		in >> entries[i].fitness;
		entries[i].chromosome = Chromosome::read(in, ap);
		if (entries[i].chromosome == NULL) {
			in.close();
			return false;
		}
	}
	return true;
}

bool Population::save(const char* filename, const AllelePool& ap) const {
	ofstream out(filename);
	if (!out || out.is_open() == 0)
		return false;

	StreamFormat fmt;
	out << fmt << entries.size() << endl;

	for (size_t i = 0; i < entries.size(); i++) {
		out << entries[i].fitness << " ";
		entries[i].chromosome->write(out, ap);
	}
	bool retVal = !out.fail() && !out.bad();
	out.close();
	return retVal;
}

bool Population::sortFunc(const entry& a, const entry& b) {
	return a.fitness > b.fitness;
}
