/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _POPULATION_H_
#define _POPULATION_H_


#include <iostream>
#include "Chromosome.h"


//
// Definition for a population of Chromosomes.
//

class Population {
public:
	struct entry {
		entry();

		Chromosome *chromosome;
		double fitness;
	};

	Population(size_t size = 0);
	~Population();

#ifndef NDEBUG
	entry& operator[](size_t index) throw(std::out_of_range);
	const entry& operator[](size_t index) const throw(std::out_of_range);
#else
	entry& operator[](size_t index);
	const entry& operator[](size_t index) const;
#endif
	size_t size() const;

	// sorts members by decending fitness
	void sort();
	void resize(size_t size);

	bool load(const char* filename, const AllelePool& ap);
	bool save(const char* filename, const AllelePool& ap) const;

private:
	// sort comparison function
	static bool sortFunc(const entry& a, const entry& b);

	std::vector<entry> entries;
};


inline Population::entry::entry()
: chromosome(NULL), fitness(0) {
}

inline Population::Population(size_t size)
: entries(size) {
}

#ifndef NDEBUG
inline Population::entry& Population::operator[](size_t index) throw(std::out_of_range) {
	return entries.at(index);
}

inline const Population::entry& Population::operator[](size_t index) const throw(std::out_of_range) {
	return entries.at(index);
}
#else
inline Population::entry& Population::operator[](size_t index) {
	return entries[index];
}

inline const Population::entry& Population::operator[](size_t index) const {
	return entries[index];
}
#endif

inline size_t Population::size() const {
	return entries.size();
}

inline void Population::resize(size_t size) {
	entries.resize(size);
}


#endif
