/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <algorithm>
#include "PopulationMask.h"


using namespace std;


PopulationMask::PopulationMask(Population& pop, bool full)
: parent(&pop) {
	if (full) {
		indices.resize(pop.size());
		for (size_t i = 0; i < pop.size(); i++)
			indices[i] = i;
	}
}

PopulationMask& PopulationMask::operator=(const PopulationMask& pm) {
	if (this != &pm) {
		parent = pm.parent;
		indices = pm.indices;
	}
	return *this;
}

void PopulationMask::sort() {
	std::sort(indices.begin(), indices.end());
}

// PopulationMask must already be sorted
void PopulationMask::computeRanks() {
	ranks.resize(size());
	size_t i = 0, j;
	while (i < size()) {
		j = i + 1;
		while (j < size() && (*this)[i].fitness == (*this)[j].fitness)
			j++;
		ranks[i] = (i + j - 1) + 2;
		i++;
		while (i < j) {
			ranks[i] = ranks[i-1];
			i++;
		}
	}
}
