/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _POPULATIONMASK_H_
#define _POPULATIONMASK_H_


#include <sstream>
#include "Population.h"


//
// Definition of a multiset of a Population
//

class PopulationMask {
public:
	PopulationMask(Population& pop, bool full = false);

#ifndef NDEBUG
	Population::entry& operator[](size_t index) throw(std::out_of_range);
	const Population::entry& operator[](size_t index) const throw(std::out_of_range);
#else
	Population::entry& operator[](size_t index);
	const Population::entry& operator[](size_t index) const;
#endif
	// Returns 2 * rank
	size_t rank(size_t index) throw(std::out_of_range);
	size_t size() const;

	std::vector<size_t>::iterator begin();
	std::vector<size_t>::iterator end();

	// Add entry at Population index
	void add(size_t index) throw(std::out_of_range);
	// Remove entry at PopulationMask index
	void remove(size_t index) throw(std::out_of_range);
	void clear();
	void reserve(size_t size);
	PopulationMask& operator=(const PopulationMask& pm);
	void sort();
	void computeRanks();

private:
	Population *parent;
	std::vector<size_t> indices;
	std::vector<size_t> ranks;
};


#ifndef NDEBUG
inline Population::entry& PopulationMask::operator[](size_t index) throw(std::out_of_range) {
	return (*parent)[indices.at(index)];
}

inline const Population::entry& PopulationMask::operator[](size_t index) const throw(std::out_of_range) {
	return (*parent)[indices.at(index)];
}
#else
inline Population::entry& PopulationMask::operator[](size_t index) {
	return (*parent)[indices[index]];
}

inline const Population::entry& PopulationMask::operator[](size_t index) const {
	return (*parent)[indices[index]];
}
#endif

inline size_t PopulationMask::rank(size_t index) throw(std::out_of_range) {
	return ranks.at(index);
}

inline size_t PopulationMask::size() const {
	return indices.size();
}

inline std::vector<size_t>::iterator PopulationMask::begin() {
	return indices.begin();
}

inline std::vector<size_t>::iterator PopulationMask::end() {
	return indices.end();
}

inline void PopulationMask::add(size_t index) throw(std::out_of_range) {
	if (index >= parent->size()) {
		std::stringstream ss;
		ss << "PopulationMask::add() out_of_range: " << index;
		throw std::out_of_range(ss.str());
	}
	indices.push_back(index);
}

inline void PopulationMask::remove(size_t index) throw(std::out_of_range) {
	if (index >= parent->size()) {
		std::stringstream ss;
		ss << "PopulationMask::remove() out_of_range: " << index;
		throw std::out_of_range(ss.str());
	}
	indices.at(index) = indices.back();
	indices.pop_back();
}

inline void PopulationMask::clear() {
	indices.clear();
}

inline void PopulationMask::reserve(size_t size) {
	indices.reserve(size);
}


#endif
