/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include "Rosenbrock.h"


double Rosenbrock::evaluate(const Chromosome& c) {
	double x = getX(c);
	double y = getY(c);
	double tmp1 = y - x * x, tmp2 = x - 1;
	return -(100 * tmp1 * tmp1 + tmp2 * tmp2);
}

double Rosenbrock::getX(const Chromosome& c) const {
	size_t len = c.size()/2;
	double d = 2, x = -2.0;
	for (size_t i = 0; i < len; i++) {
		if (c[i] == ON)
			x += d;
		d /= 2;
	}
	return x;
}

double Rosenbrock::getY(const Chromosome& c) const {
	double d = 2, y = -2.0;
	for (size_t i = c.size()/2; i < c.size(); i++) {
		if (c[i] == ON)
			y += d;
		d /= 2;
	}
	return y;
}
