/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include "TrapMax.h"


using namespace std;


bool TrapMax::init(ParameterSet& ps, const AllelePool& ap) {
	BinaryEvolutionTask::init(ps, ap);
	if (!ps.get("trapSize", trapSize)) {
		cerr << "Error: Required parameter 'trapSize' missing" << endl;
		return false;
	}
	return true;
}

double TrapMax::evaluate(const Chromosome& c) {
	assert(c.size()%trapSize == 0);
	double sum = 0;
	size_t i = 0;
	while (i < c.size()) {
		int lsum = 0;
		for (size_t j = 0; j < static_cast<size_t>(trapSize); j++,i++)
			if (c[i] == ON)
				lsum++;
		if (lsum == trapSize || lsum == 0)
			sum += trapSize - lsum;
		else
			sum += lsum;
	}
	return sum;
}
