/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <algorithm>
#include <vector>
#include "Uniform.h"


using namespace std;


bool Uniform::initAlleles(const ParameterSet& ps, AllelePool& ap) {
	if (!ps.get("alleles", alleles)) {
		cerr <<	"Error: required parameter \"alleles\" missing" << endl;
		return false;
	}
	if (alleles <= 1) {
		cerr << "Error: alleles <= 1" << endl;
		return false;
	}
	if (alleles > 26) {
		cerr << "Error: alleles > 26" << endl;
		return false;
	}

	string allele("a");
	int count = alleles;
	do {
		ap.newAllele(allele);
		allele[0]++;
	} while (--count > 0);

	return true;
}

bool Uniform::init(ParameterSet& ps, const AllelePool& ap) {
	alleles = ap.size();
	return true;
}

double Uniform::evaluate(const Chromosome& c) {
	vector<size_t> occ(alleles);
	for (size_t i = 0; i < static_cast<size_t>(alleles); i++)
		occ[i] = 0;
	for (size_t i = 0; i < c.size(); i++)
		occ[c[i]]++;
	return *max_element(occ.begin(), occ.end());
}
