/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _VECTORMAP_H_
#define _VECTORMAP_H_


#include <map>
#include <vector>


namespace std {

template <class T> struct less<vector<T> > {
	bool operator()(const vector<T>& x, const vector<T>& y) const;
};


template <class T> bool less<vector<T> >::operator()(const vector<T>& x, const vector<T>& y) const {
        assert(x.size() == y.size());
        for (size_t i = 0; i < x.size(); i++)
                if (x[i] < y[i])
                        return true;
                else if (x[i] > y[i])
                        return false;
        return false;
}

}


template <class T, class U> class VectorMap : public std::map<std::vector<T>, U, std::less<std::vector<T> > > {
private:
	typedef std::map<std::vector<T>, U, std::less<std::vector<T> > > parent_class;

public:
	U& get(const std::vector<T>& key);
};


template <class T, class U> inline U& VectorMap<T,U>::get(const std::vector<T>& key) {
	typename parent_class::iterator it = find(key);
	if (it != parent_class::end())
		return it->second;
	return insert(make_pair(key, U())).first->second;
}


#endif
