/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _MULTITYPE_H_
#define _MULTITYPE_H_


#include <stdexcept>
#include <typeinfo>
#include <string>


class Multitype {
public:
	enum TYPE {UNKNOWN, LONG, DOUBLE, STRING};

	Multitype();
	Multitype(long l);
	Multitype(double d);
	Multitype(const char* s);
	Multitype(const std::string& s);
	Multitype(const Multitype& m);
	~Multitype();

	TYPE getType() const;

	Multitype& operator=(long l) throw(std::invalid_argument);
	Multitype& operator=(double d) throw(std::invalid_argument);
	Multitype& operator=(const char* s) throw(std::invalid_argument);
	Multitype& operator=(const std::string& s) throw(std::invalid_argument);
	Multitype& operator=(const Multitype& m) throw(std::invalid_argument);
	void markUnknown();

	operator const long&() const throw(std::bad_cast);
	operator long&() throw(std::bad_cast);
	operator const double&() const throw(std::bad_cast);
	operator double&() throw(std::bad_cast);
	operator const std::string&() const throw(std::bad_cast);
	operator std::string&() throw(std::bad_cast);

	friend std::ostream& operator<<(std::ostream& out, const Multitype& m);

private:
	bool parseLong();
	bool parseDouble();

	TYPE type;
	union {
		long as_long;
		double as_double;
	};
	std::string *as_string;
};


#endif
