/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _PARAMETERFILE_H_
#define _PARAMETERFILE_H_


#include "ParameterSet.h"


class ParameterFile {
public:
	typedef std::map<std::string,ParameterSet>::iterator iterator;
	typedef std::map<std::string,ParameterSet>::const_iterator const_iterator;

	ParameterFile();
	ParameterFile(const ParameterFile& pf);
	ParameterFile(const char* filename);

	size_t size() const;
	bool contains(const std::string& section) const;
	iterator find(const std::string& section);
	const_iterator find(const std::string& section) const;
	ParameterSet& operator[](const std::string& section);
	const ParameterSet& operator[](const std::string& section) const throw(std::invalid_argument);
	bool get(const std::string& section, ParameterSet& dest) const;
	iterator begin();
	const_iterator begin() const;
	iterator end();
	const_iterator end() const;

	void remove(const std::string& section);
	void clear();

	bool load(const char* filename);
	friend std::istream& operator>>(std::istream& in, ParameterFile& pf);
	bool save(const char* filename) const;
	friend std::ostream& operator<<(std::ostream& out, const ParameterFile& pf);

private:
	std::map<std::string,ParameterSet> sets;
};


inline bool ParameterFile::load(const char* filename) {
	return ::load(filename, this);
}

inline bool ParameterFile::save(const char* filename) const {
	return ::save(filename, this);
}


#endif
