/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _PARAMETERSET_H_
#define _PARAMETERSET_H_


#include <map>
#include <stdexcept>
#include "Multitype.h"
#include "util.h"


class ParameterSet {
public:
	typedef std::map<std::string,Multitype>::iterator iterator;
	typedef std::map<std::string,Multitype>::const_iterator const_iterator;

	ParameterSet();
	ParameterSet(const ParameterSet& ps);
	ParameterSet(const char* filename);

	size_t size() const;
	bool contains(const std::string& name) const;
	iterator find(const std::string& name);
	const_iterator find(const std::string& name) const;
	Multitype& operator[](const std::string& name);
	const Multitype& operator[](const std::string& name) const throw(std::invalid_argument);
	bool get(const std::string& name, long& dest) const;
	bool get(const std::string& name, double& dest) const;
	bool get(const std::string& name, std::string& dest) const;
	iterator begin();
	const_iterator begin() const;
	iterator end();
	const_iterator end() const;

	void remove(const std::string& name);
	void clear();
	void add(const ParameterSet& ps);

	bool load(const char* filename);
	friend std::istream& operator>>(std::istream& in, ParameterSet& ps);
	bool save(const char* filename) const;
	friend std::ostream& operator<<(std::ostream& out, const ParameterSet& ps);

private:
	std::map<std::string,Multitype> values;
};


inline bool ParameterSet::load(const char* filename) {
	return ::load(filename, this);
}

inline bool ParameterSet::save(const char* filename) const {
	return ::save(filename, this);
}


#endif
