/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#ifndef _UTIL_H_
#define _UTIL_H_


#include <cstdlib>
#include <fstream>
#include <iostream>
#include <string>
#include <sys/stat.h>
#include "exceptions.h"


// const long double PI = 3.1415926535897932384626433832795029;
// standard round towards zero
// double round(double n);

inline int abs(int x) { return x >= 0 ? x : x; } 

// discards characters until non-whitespace (exclusive) or newline (inclusive)
// is found
std::istream& wsline(std::istream& in);
// discards characters until newline (inclusive) is found
std::istream& endl(std::istream& in);
// damn silly hack for Condor
std::istream& detecteof(std::istream& in);
std::istream& getline(std::istream& in, std::ostream& out) throw(io_error);
std::istream& skipLine(std::istream& in, int numLines = 1);
std::string& trim(std::string& str);
void status(std::ios& subject, std::ostream& out);

//std::istream& operator>>(std::istream& in, const char& c);

bool exists(const char* path);
bool readable(const char* path);
bool isDirectory(const char* path);
// like mkdir but will create parent directories
bool makedir(const char* path, mode_t mode = S_IRWXU | S_IRWXG | S_IRWXO);
// wrapper for mkstemp returning temporary filename
std::string maketemp();
// wrapper for mkdtemp returning temporary directory
std::string maketempdir();
// returns the directory portion of a path
std::string pathDirectory(const char* path);
// returns the file portion of a path
std::string pathFilename(const char* path);
// returns the file basename portion of a path
std::string pathBase(const char* path);
// returns the file extension portion of a path
std::string pathExtension(const char* path);
// returns the path with the file extension removed,
// a.k.a. pathDirectory + pathBase
std::string pathTrimExtension(const char* path);

template <class T> bool load(const char* filename, T* t) {
	std::ifstream in(filename);
	if (!in || !in.is_open())
		return false;
	in >> *t;
	bool retVal = !in.fail() && !in.bad();
	in.clear();
	in.close();
	return retVal;
}

template <class T> bool save(const char* filename, const T* t) {
	std::ofstream out(filename);
	if (!out || !out.is_open())
		return false;
	out << *t;
	bool retVal = out.good();
	out.clear();
	out.close();
	return retVal;
}

class StreamFormat {
public:
	StreamFormat();

	friend std::ostream& operator<<(std::ostream& out, StreamFormat& fmt);

private:
	bool store;
	std::ios::fmtflags flags;
	std::streamsize precision;
};


#endif
