/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <cmath>
#include <cstring>
#include <gsl/gsl_cdf.h>
#include <fstream>
#include <iostream>
#include <list>
#include "util.h"


using namespace std;


struct point {
	double x, cp, error;
};


const int maxDoF = 50;
const double chiPoints[] = {0.005, 0.01, 0.05, 0.1, 0.15, 0.2, 0.25, 0.35, 0.45, 0.55, 0.65, 0.75, 0.85, 0.9, 0.95, 0.99, 0.995};
const double gaussPoints[] = {0.55, 0.6, 0.65, 0.75, 0.85, 0.9, 0.95, 0.99, 0.995};


void showUsage();


int main(int argc, char* argv[]) {
	if (argc > 1) {
		showUsage();
		if (strcmp(argv[1], "--help") == 0)
			return 0;
		return 1;
	}

	ofstream out("chi2.dat");
        if (!out || !out.is_open()) {
                cerr << "Error: unable to open 'chi2.dat'" << endl;
                return 1;
        }
	StreamFormat sf;
	out << sf << noshowpoint << maxDoF << endl;

	for (int i = 1; i <= maxDoF; i++) {
		int numPoints = sizeof(chiPoints) / sizeof(double);
		out << numPoints << endl;
		for (size_t j = 0; j < numPoints; j++)
			out << gsl_cdf_chisq_Pinv(chiPoints[j], i) << ' ' << chiPoints[j] << endl;
	}

	int numPoints = sizeof(gaussPoints) / sizeof(double);
	out << numPoints << endl;
	for (size_t j = 0; j < numPoints; j++)
		out << gsl_cdf_ugaussian_Pinv(gaussPoints[j]) << ' ' << gaussPoints[j] << endl;

	out.close();

	return 0;
}

void showUsage() {
	cout << "Usage: makechi2" << endl << endl;
	cout << "Writes the file chi2.dat to the present working directory." << endl << endl;
}
