/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <cstring>
#include <iostream>
#include <fstream>
#include <sstream>
#include "Random.h"


using namespace std;


double rngOne();
double rngGauss();
void showUsage();


int main(int argc, char* argv[]) {
	if (argc < 4 || argc > 5) {
		showUsage();
		return 1;
	} else if (strcmp(argv[1], "--help") == 0) {
		showUsage();
		return 0;
	}

	if (strcmp(argv[1], "-one") != 0 && strcmp(argv[1], "-gauss") != 0) {
		cerr << "Error: unknown coefficient type '" << argv[1] << "'" << endl;
		return 1;
	}
	double (*rng)();
	if (strcmp(argv[1], "-one") == 0)
		rng = rngOne;
	else
		rng = rngGauss;
	stringstream ss;
	ss << argv[2];
	long width = -1;
	ss >> width;
	ss.clear();
	ss.str("");
	ss << argv[3];
	long height = -1;
	ss >> height;
	if (width < 3 || height < 3) {
		cerr << "Error: Width and height must be at least 3." << endl;
		return 1;
	}

	ofstream fout;
	ostream *out;
	if (argc == 5) {
		out = &fout;
		fout.open(argv[4]);
		if (!fout.good() || !fout.is_open()) {
			cerr << "Error: Unable to open \"" << argv[4] << "\"" << endl;
			return 1;
		}
	} else
		out = &cout;
	out->precision(16);

	(*out) << width << ' ' << height << endl;
	long pos = 0;
	for (long h = 0; h < height; h++)
		for (long w = 0; w < width; w++) {
			// Up
			long i = w + width * ((h + height - 1) % height);
			if (i > pos)
				(*out) << i << ' ' << rng() << ' ';
			// Left
			i = ((w + width - 1) % width) + width * h;
			if (i > pos)
				(*out) << i << ' ' << rng() << ' ';
			// Right
			i = ((w + 1) % width) + width * h;
			if (i > pos)
				(*out) << i << ' ' << rng() << ' ';
			// Down
			i = w + width * ((h + 1) % height);
			if (i > pos)
				(*out) << i << ' ' << rng() << ' ';
			(*out) << endl;
			pos++;
		}
	fout.close();

	return 0;
}

double rngOne() {
	return 2 * static_cast<long>(g_Random.next32(0, 1)) - 1;
}

double rngGauss() {
	return g_Random.nextGaussian();
}

void showUsage() {
	cout << "Usage: makespinglass -one <width> <height> [<file>]" << endl;
	cout << "       makespinglass -gauss <width> <height> [<file>]" << endl;
	cout << endl << "In the first instance, coupling constants for adjacent spins will be" << endl;
	cout << "uniformly drawn from the set {-1, +1}.  In the second instance, coupling" << endl;
	cout << "constants will be drawn from the standard normal distribution." << endl << endl;
}
