/***********************************************************************
 * Copyright 2009, Matthew Alden
 *
 * This file is part of MARLEDA.
 *
 * MARLEDA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MARLEDA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MARLEDA.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/

#include <cstring>
#include <fstream>
#include <sstream>
#include "util.h"


using namespace std;


void showUsage();


int main(int argc, char* argv[]) {
	if (argc < 2 || strcmp (argv[1], "--help") == 0) {
		showUsage();
		return 1;
	}

	ifstream in(argv[1]);
	if (!in.good() || !in.is_open()) {
		cerr << "Unable to open " << argv[1] << endl;
		return 1;
	}

	size_t width, height;
	in >> width >> height >> endl >> detecteof;
	if (width != height) {
		cerr << "Error: SGGSS is limited to square lattices." << endl;
		in.close();
		return 1;
	}

	// First pass, detect "pm" or "gauss" model
	bool gaussModel = false;
	size_t spin2;
	double coupling;
	stringstream ss;
	while (in.good() && !gaussModel) {
		ss.clear();
		getline(in, ss);
		ss >> detecteof;
		while (ss.good()) {
			ss >> spin2 >> coupling >> ws >> detecteof;
			if (coupling != -1 && coupling != 1) {
				gaussModel = true;
				break;
			}
		}
	}
	in.close();

	// Pass two, converting the data
	in.open(argv[1]);
	if (!in.good() || !in.is_open()) {
		cerr << "Unable to open " << argv[1] << endl;
		return 1;
	}
	in >> width >> height >> endl >> detecteof;

	ofstream out;
	string outname;
	if (argc > 2)
		outname = argv[2];
	else
		outname = pathTrimExtension(argv[1]) + ".sggss";
	out.open(outname.c_str());
	if (!out.good() || !out.is_open()) {
		cerr << "Unable to open " << outname.c_str() << endl;
		in.close();
		return 1;
	}
	out.precision(16);
	
	if (gaussModel)
		out << "type: gauss" << endl;
	else
		out << "type: pm" << endl;
	out << "size: " << width << endl;
	out << "name: " << argv[1] << endl << endl;
	size_t spin1 = 0;
	while (in.good()) {
		ss.clear();
		getline(in, ss);
		ss >> detecteof;
		while (ss.good()) {
			ss >> spin2 >> coupling >> ws >> detecteof;
			out << spin1+1 << ' ' << spin2+1 << ' ' << coupling << endl;
		}
		spin1++;
	}
	in.close();
	out.close();

	return 0;
}

void showUsage() {
	cout << "Usage: spin2sggss <spin-file> [<output-file>]" << endl;
}
